# -*- coding: utf-8 -*-
"""
/***************************************************************************
 gridTool
                                 A QGIS plugin
 gridTool
                             -------------------
        begin                : 2016-07-22
        copyright            : (C) 2016 by GeoLog / www.geolog.kr
        email                : kju7755@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4 import uic

from qgis.core import *
from qgis.gui import *
from qgis.utils import *

import math
import ctypes

MessageBox = ctypes.windll.user32.MessageBoxW

class gridTool(QgsMapTool):   
    

    def __init__(self, canvas, parent = None):

        QgsMapTool.__init__(self, canvas)
        
        self.canvas = canvas
        self.mapPoint = None

        #self.cursor = None
        #self.cursor = QCursor(Qt.PointingHandCursor)
            
        self.rbGrid = None
        self.jobFlag = None
        self.cursor = None
        self.mousePressed = False
        self.prsPnt = None
        
        self.mapRect = None
        self.grdOrgPnt = None
        self.prsGrdOrgPnt = None
        
        self.grdWidth = None
        self.grdHeight = None
        
        self.dicSelectedCell = {}
        self.colCnt = 0    #가로 Cell 갯수
        self.rowCnt = 0    #세로 Cell 갯수
        
    def activate(self):
        pass

        
    def canvasPressEvent(self, event):
        self.mousePressed = True
        self.prsPnt = self.toMapCoordinates(event.pos())
        self.prsGrdOrgPnt = self.grdOrgPnt
            
        if self.jobFlag == 'moveGrid':
            self.cursor = QCursor(Qt.ClosedHandCursor)
            self.canvas.setCursor(self.cursor)
            
    def canvasMoveEvent(self, event):
        
        self.canvas.setCursor(self.cursor)
        
        if self.mousePressed and self.jobFlag == 'moveGrid' \
            and self.prsPnt and self.grdOrgPnt:

            movePnt = self.toMapCoordinates(event.pos())

            self.grdOrgPnt = QPoint(self.prsGrdOrgPnt.x() + movePnt.x()-self.prsPnt.x(), \
                                    self.prsGrdOrgPnt.y() + movePnt.y()-self.prsPnt.y())
            
            self.plotGrid(False, self.mapRect, self.sheetWidht, self.sheetHeight, self.plotScale)
                           
    def canvasReleaseEvent(self, event):
        
        rlsPnt = self.toMapCoordinates(event.pos())
        
        if self.jobFlag == 'selectGrid' and self.rbGrid:
            
            col, row = self.mouseHoverCell(rlsPnt)
            if col == 0 or row == 0:
                return
            
            cr = '%d,%d' % (col, row)
            if self.dicSelectedCell.has_key(cr):
                rb = self.dicSelectedCell.get(cr)
                self.canvas.scene().removeItem(rb)
                del self.dicSelectedCell[cr]
                
            else:
                self.addSelectDic(col, row)

        elif self.jobFlag == 'moveGrid' and self.rbGrid:    #선택되어진 러버벤드 이동
            self.cursor = QCursor(Qt.OpenHandCursor)
            self.canvas.setCursor(self.cursor)
            self.moveSelectCell(rlsPnt)

            
        self.mousePressed = False
        self.prsPnt = None
               
                                                          
    def addSelectDic(self, col, row):
        btmLeftPnt = QgsPoint(self.grdOrgPnt.x() + (col-1)*self.grdWidth,
                              self.grdOrgPnt.y() + (row-1)*self.grdHeight)
        topRightPnt = QgsPoint(self.grdOrgPnt.x() + col*self.grdWidth,
                              self.grdOrgPnt.y() + row*self.grdHeight)
                
        rect = QgsRectangle(btmLeftPnt, topRightPnt)
        rectGeom =  QgsGeometry.fromRect(rect)
        rbCell = QgsRubberBand(self.canvas, QGis.Polygon)
        rbCell.setColor(Qt.gray)
        rbCell.setBrushStyle(Qt.Dense6Pattern)
        rbCell.setWidth(0.1)                
        rbCell.addGeometry(rectGeom, None)
        
        cr = '%d,%d' % (col, row)
        self.dicSelectedCell[cr] = rbCell

 
    def moveSelectCell(self, rlsPnt):
        for cr, rbCell in self.dicSelectedCell.iteritems():
            self.canvas.scene().removeItem(rbCell)
            del self.dicSelectedCell[cr]
                            
            col, row = cr.split(',')
            self.addSelectDic(int(col), int(row))

        
    def plotGrid(self, newGrid, mapRect, sheetWidht, sheetHeight, plotScale):
        #그려진 선택 셀 삭제
        if newGrid:
            for cr, rbCell in self.dicSelectedCell.iteritems():
                self.canvas.scene().removeItem(rbCell)
            self.dicSelectedCell = {}
                    
        if self.rbGrid:
            self.rbGrid.reset(QGis.Line)  
        else:      
            self.rbGrid = QgsRubberBand(self.canvas, QGis.Line)
            self.rbGrid.setColor(Qt.black)
            #self.rbGrid.setBrushStyle(Qt.NoBrush)
            self.rbGrid.setWidth(0.1)

        self.mapRect = mapRect
        
        self.grdWidth = sheetWidht/1000*plotScale
        self.grdHeight = sheetHeight/1000*plotScale
        
        self.sheetWidht = sheetWidht
        self.sheetHeight = sheetHeight
        self.plotScale = plotScale
        
        self.colCnt = int(mapRect.width() / self.grdWidth) + 1
        self.rowCnt = int(mapRect.height() / self.grdHeight) + 1
        
        topLimit = self.grdOrgPnt.y() + self.rowCnt * self.grdHeight
        ritLimit = self.grdOrgPnt.x() + self.colCnt * self.grdWidth
        
        i = 0
        while True:

            #세로선
            if i <= self.colCnt:
                btnPnt = QgsPoint(self.grdOrgPnt.x() + i*self.grdWidth, self.grdOrgPnt.y())
                topPnt = QgsPoint(self.grdOrgPnt.x() + i*self.grdWidth, topLimit)
                self.rbGrid.addGeometry(QgsGeometry.fromPolyline([btnPnt, topPnt]), None)
            
            #가로선
            if i <= self.rowCnt:
                lftPnt = QgsPoint(self.grdOrgPnt.x(), self.grdOrgPnt.y() + i*self.grdHeight)
                ritPnt = QgsPoint(ritLimit, self.grdOrgPnt.y() + i*self.grdHeight)
                self.rbGrid.addGeometry(QgsGeometry.fromPolyline([lftPnt, ritPnt]), None)
                        
            i += 1
            if i > self.colCnt and i > self.rowCnt:
                break
                
    def selectGrid(self):
        self.jobFlag = 'selectGrid'
        self.cursor = QCursor(Qt.PointingHandCursor)
        
    def moveGrid(self):
        self.jobFlag = 'moveGrid'
        self.cursor = QCursor(Qt.OpenHandCursor)
         
    def mouseHoverCell(self, point):
        '''
        self.grdOrgPnt = 전체 그리드 외곽 좌측하단 좌표
        self.grdWidth  = 도곽 폭
        '''
                
        col = 0
        row = 0
        
        while True:
            if (self.grdOrgPnt.x() + col*self.grdWidth) > point.x() \
                or col*self.grdWidth > self.grdOrgPnt.x() \
                or self.grdOrgPnt.x()+self.colCnt*self.grdWidth < point.x():
                break
            col += 1       

        while True:
            if (self.grdOrgPnt.y() + row*self.grdHeight) > point.y() \
                or row*self.grdHeight > self.grdOrgPnt.y() \
                or self.grdOrgPnt.y()+self.rowCnt*self.grdHeight < point.y():
                break            
            row += 1
            
        return col, row
                    

    
    