﻿# -*- coding: utf-8 -*-
"""
/***************************************************************************
 pnlPlot
                                 A QGIS plugin
 pnlPlot
                             -------------------
        begin                : 2016-07-21
        copyright            : (C) 2016 by GeoLog / www.geolog.kr
        email                : kju7755@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4.QtXml import QDomDocument

from qgis.core import *
from qgis.gui import *
from qgis import utils

import ctypes
from glob import glob
import os.path
from time import gmtime, strftime

import os, sys
#from win32com import client

#import win32api
import win32print
import subprocess

sys.path.append(os.path.dirname(__file__) + "../PoSS/psManagement")

from ui_pnlPlot import Ui_pnlPlot
from gridTool import gridTool

MessageBox = ctypes.windll.user32.MessageBoxW
templatePath = os.path.dirname(__file__) + "/template"
printPath = os.path.dirname(__file__) + "/map"
lstTempPath = []

        
class pnlPlot(QDockWidget, Ui_pnlPlot):
    def __init__(self, parent=None):
        QDockWidget.__init__(self)
        
        self.setupUi(self)
        self.parent = parent
        
        self.canvas = parent.mapCanvas()

        self.btnGridDraw.clicked.connect(lambda: self.gridDraw())
        self.btnGridDelete.clicked.connect(lambda: self.gridDelete())
        self.btnGridSelect.clicked.connect(lambda: self.gridSelect())
        self.btnGridSelectAll.clicked.connect(lambda: self.gridSelectAll())
        self.btnGridSelectClear.clicked.connect(lambda: self.gridSelectClear())
        self.btnGridMove.clicked.connect(lambda: self.gridMove())
        self.btnPrintMap.clicked.connect(lambda: self.printMap())
        self.btnTemplateRefresh.clicked.connect(lambda: self.templateRefresh())
        self.btnPdfFolder.clicked.connect(lambda: self.selectPdfFolder())
        self.btnSetPrinter.clicked.connect(lambda: self.setPrinterOption())
        
        self.hrtLayout = QVBoxLayout(self.frmMessage)
        
        self.iniPanel()
        self.composer = None

        self.gridTool = gridTool(self.canvas)

        self.setObjectName('pnlPossPlot')

        parent.addDockWidget(Qt.RightDockWidgetArea, self)
        
        self.cboTemplate.currentIndexChanged.connect(self.gridDelete)
        
        self.installEventFilter(self)
        

    def eventFilter(self, qobject, qevent):

        qtype = qevent.type()
        
        if QEvent == None:
            return

        if qtype == QEvent.Hide:
            self.gridClear()
            return True

        return super(pnlPlot, self).eventFilter(qobject, qevent)
         

        
    def iniPanel(self):
        settings = QSettings("PoSS", "possPlot")
        self.txtTitle.setText(settings.value('title', u'도 면 출 력'))
        self.txtPdfFolder.setText(settings.value('pdfPath', printPath))
        if settings.value('pdfCheck', 'true') == 'true':
            self.radToPdf.setChecked(True)
        else:
            self.radToPrinter.setChecked(True)

        self.listScale()
        self.listTemplate(settings.value('template', ''))
        self.listPrinter(settings.value('printerName', ''))

    def listScale(self):
        self.cboScale.addItem('500')
        self.cboScale.addItem('1000')
        self.cboScale.addItem('5000')
        self.cboScale.addItem('10000')
        self.cboScale.addItem('20000')
        self.cboScale.addItem('50000')
        
    def listTemplate(self, templateName):
        global lstTempPath
        
        lstFile = glob(templatePath + '\*.qpt')
        self.cboTemplate.clear()
        lstTempPath = []
        for tmp in lstFile:
            lstTempPath.append(tmp)
            self.cboTemplate.addItem(os.path.basename(tmp))
            
        index = self.cboTemplate.findText(templateName, Qt.MatchFixedString)
        if index >= 0:
            self.cboTemplate.setCurrentIndex(index)
        else:            
            self.cboTemplate.setCurrentIndex(0)
            
    def listPrinter(self, printerName):
        printers = win32print.EnumPrinters(win32print.PRINTER_ENUM_LOCAL, None, 2)
        for printer in printers :
            self.cboPrinter.addItem(printer['pPrinterName'])
                               
        index = self.cboPrinter.findText(printerName, Qt.MatchFixedString)
        if index >= 0:
            self.cboPrinter.setCurrentIndex(index)
        else:
            self.cboPrinter.setCurrentIndex(0)
                 
    def gridDraw(self):
        self.composer = None
        self.getComposer()
        sheetWidth, sheetHeight = self.getSheetSize()
        
        if sheetWidth > 0 and sheetHeight > 0:
            mapRect = self.canvas.extent()
            mapOrgX = mapRect.xMinimum()
            mapOrgY = mapRect.yMinimum()
            
            self.gridTool.grdOrgPnt = QgsPoint(mapOrgX, mapOrgY)
            
            gridTool.plotGrid(self.gridTool, True, mapRect,\
                              sheetWidth, sheetHeight, float(self.cboScale.currentText()))

    def gridClear(self):
        try:
            for cr, rbCell in self.gridTool.dicSelectedCell.iteritems():
                self.canvas.scene().removeItem(rbCell)
                
            self.gridTool.dicSelectedCell = {}
            
            if self.gridTool.rbGrid:
                self.gridTool.rbGrid.reset()    
                self.gridTool.rbGrid = None
            
            self.gridTool.grdOrgPnt = None
        
        except:
            pass

    def gridDelete(self):
        self.gridClear()
        
    def gridSelect(self):
        self.canvas.setMapTool(self.gridTool)
        gridTool.selectGrid(self.gridTool)
        
    def gridSelectAll(self):

        for row in range(1, self.gridTool.rowCnt+1):
            for col in range(1, self.gridTool.colCnt+1):
                
                cr = '%d,%d' % (col, row)
                if self.gridTool.dicSelectedCell.has_key(cr):
                    pass
                else:                              
                    self.gridTool.addSelectDic(col, row)
        
    def gridSelectClear(self):
        for row in range(1, self.gridTool.rowCnt+1):
            for col in range(1, self.gridTool.colCnt+1):       
                cr = '%d,%d' % (col, row)
                if self.gridTool.dicSelectedCell.has_key(cr):        
                    rb = self.gridTool.dicSelectedCell.get(cr)
                    self.canvas.scene().removeItem(rb)
                    del self.gridTool.dicSelectedCell[cr]
                            
        self.gridTool.dicSelectedCell = {}
        
    def gridMove(self):
        self.canvas.setMapTool(self.gridTool)
        gridTool.moveGrid(self.gridTool)

    def getComposer(self):
        try:
            tmpFilePath = lstTempPath[self.cboTemplate.currentIndex()]
            tmpFile = file(tmpFilePath)
            tmpCont = tmpFile.read()
            tmpFile.close()
            
            doc = QDomDocument()
            doc.setContent(tmpCont)
           
            self.composer = QgsComposition(self.canvas.mapSettings())
            self.composer.loadFromTemplate(doc)

        except Exception as e:
            print e.message
            
        finally:
            return
                        
    def getSheetSize(self):
        if self.composer:
            mapItem = self.composer.getComposerItemById('map')
            return mapItem.rect().width(), mapItem.rect().height()  #mm
        else:
            return 0, 0
                        
    def selectPdfFolder(self):
        dialog = QFileDialog()      
        dialog.setFileMode(QFileDialog.Directory)
        dialog.setOption(QFileDialog.ShowDirsOnly)
        dialog.setWindowTitle(u"PDF 출력 폴더 선택")

        if dialog.exec_():
            folderPath = dialog.selectedFiles()[0]
            self.txtPdfFolder.setText(folderPath)
    
    def printMap(self):  
        if self.radToPdf.isChecked() == True:
            self.printMapPDF()
        else:
            self.printMapPrinter()
            
    def printMapPDF(self):        
        try:
            QApplication.setOverrideCursor(Qt.WaitCursor)
            
            if self.txtPdfFolder.text() != '':
                pdfFolder = self.txtPdfFolder.text()
            else:     
                pdfFolder = printPath
            
            if not os.path.exists(pdfFolder):
                os.makedirs(pdfFolder)   
                                    
            mapItem = self.setComposerItem()
            if mapItem == None:
                self.parent.messageBar().pushMessage(u'출력', u'composer template에 map item이 없습니다.' ,
                                                     level=QgsMessageBar.CRITICAL, duration=5)
                return          
            
            i = 0
            for cr, rbCell in self.gridTool.dicSelectedCell.iteritems():
                i += 1
                extent = rbCell.asGeometry().boundingBox()
                mapItem.zoomToExtent(extent)
                self.composer.refreshItems()

                prtPath = '%s\\map_%d.pdf' % (pdfFolder, i)
                succ = self.composer.exportAsPDF(prtPath)
                self.printMessage(succ, prtPath)
                
#                 msgLabel = QLabel(prtPath)
#                 self.hrtLayout.addWidget(msgLabel)
                
        except Exception as e:
            print e.message 
            
        finally:
            QApplication.restoreOverrideCursor()

    def printMapPrinter(self):        
        try:
            QApplication.setOverrideCursor(Qt.WaitCursor)
                                   
            mapItem = self.setComposerItem() 
            if mapItem == None:
                self.parent.messageBar().pushMessage(u'출력', u'composer template에 map item이 없습니다.' ,
                                                     level=QgsMessageBar.CRITICAL, duration=5)
                return 
            
            printer, painter = self.setPrinter(self.composer)
                
            for cr, rbCell in self.gridTool.dicSelectedCell.iteritems():
                extent = rbCell.asGeometry().boundingBox()
                mapItem.zoomToExtent(extent)
                self.composer.refreshItems()

                succ = self.composer.doPrint(printer, painter)  #printer, painter, startNewPage

            painter.end()
                    
        except Exception as e:
            print e.message 
            
        finally:
            QApplication.restoreOverrideCursor()
            
    def setComposerItem(self):
        mapItem = None
        
        try:
            mapItem = self.composer.getComposerItemById('map')
            mapItem.setMapCanvas(self.canvas)
                        
            title = self.composer.getComposerItemById('title')
            if title:
                title.setText(self.txtTitle.text())
            
            prtDate = self.composer.getComposerItemById('date')
            if prtDate:
                prtDate.setText(strftime("%Y-%m-%d", gmtime()))
               
            prtScale = self.composer.getComposerItemById('scale')
            if prtScale:
                prtScale.setText('1:%s' % self.cboScale.currentText())

        except Exception as e:
            pass
        
        return mapItem
    
    def templateRefresh(self):
        self.listTemplate(self.cboTemplate.currentText())
        
    def setPrinter(self, c):
        printer = QPrinter()
#         printer.setOutputFormat(QPrinter.PdfFormat)
#         printer.setOutputFileName("'F:\\tmp\\report.pdf'")
        printer.setPrinterName(self.cboPrinter.currentText())
        printer.setPaperSize(QSizeF(c.paperWidth(), c.paperHeight()), QPrinter.Millimeter)
        printer.setFullPage(True)
        printer.setColorMode(QPrinter.Color)
        printer.setResolution(c.printResolution())
        
        painter = QPainter(printer)
        
        return printer, painter
    
    def printMessage(self, succ, prtPath):
        if succ:
            self.parent.messageBar().pushMessage(u'출력', prtPath,
                                                 level=QgsMessageBar.INFO, duration=5) 
                    
        else:
            self.parent.messageBar().pushMessage(u'출력', u'출력이 진행되지 않았습니다.',
                                                 level=QgsMessageBar.WARNING, duration=5)        
        
    def setPrinterOption(self):
        dialog = QPrintDialog()
        if dialog.exec_() == QDialog.Accepted:
            return 
    